#! /bin/sh

# $NetBSD$

# Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


# Usage:
#
# getpkgsrc [-d=repo] [-p] [-v] category/package...
#	-d=repo 	(default repo is anoncvs@anoncvs.netbsd.org:/cvsroot)
#	-p		preserve checked out tree
#	-v		act verbsoely
#	packages should be specified as <category>/<package-name>
#
# this utility requires a bootstrapped pkgsrc, and a cvs utility

# utility function to print a message and die
die() {
	echo $* >&2
	exit 1
}

# these are the packages we need
musthaves="pkgtools/x11-links sysutils/install-sh"

# (recursive) function which checks out a single
checkout() {
	# get the package wanted
	cvs -d ${repo} co -P pkgsrc/$1
	# get any prereqs
	bl3="pkgsrc/$1/buildlink3.mk"
	if [ ! -f ${bl3} ]; then
		bl3=""
	fi
	makefile=pkgsrc/$1/Makefile
	if [ ! -f ${makefile} ]; then
		makefile=""
	fi
	if [ "${bl3}" != "" -o "${makefile}" != "" ]; then
		prereqs="$(awk '/^.include.*/ { split($2, dir, "/"); if (dir[3] != "mk") printf("%s/%s ", dir[3], dir[4])}' ${makefile} ${bl3})"
		tools="$(awk '/^[ 	]*USE_LIBTOOL=/ { print "devel/libtool-base" }' ${makefile})"
	fi
	# check the prereqs out
	for p in ${musthaves} ${prereqs} ${tools}; do
		if [ ! -d pkgsrc/${p} ]; then
			checkout ${p}
		fi
	done
}

# (recursively) get the files for the pkgsrc entry
getentry() {
	d=$1
	checkout ${d}
	# also look for tools
	tools=$(cd pkgsrc/${d} && ${make} show-var VARNAME=TOOLS_CREATE | awk '{ sub("\\[.*", ""); print }')
	for t in ${tools}; do
		tooldep=$(cd pkgsrc/$d && ${make} show-var VARNAME=TOOLS_DEPENDS.${t})
		case "${tooldep}" in
		"")	;;
		*)	getentry $(echo "${tooldep}" | awk '{ sub(".*:\\.\\./\\.\\./", ""); print }')
			;;
		esac
	done
	# wait until we have everything checked out before expanding {BUILD_,}DEPENDS
	depends=$(cd pkgsrc/$d && ${make} show-vars VARNAMES="BUILD_DEPENDS DEPENDS")
	for dep in ${depends}; do
		depdir=$(echo "${dep}" | awk '{ sub(".*:\\.\\./\\.\\./", ""); print }')
		getentry ${depdir}
	done
}

# check arguments
case $# in
0)	die "No package names given" ;;
esac

# local variables
removetree=true
repo=anoncvs@anoncvs.netbsd.org:/cvsroot

# command line arguments
while [ $# -gt 0 ]; do
	case "$1" in
	-d=*)	repo=$(echo "$1" | sed -e 's|-d=||') ;;
	-p)	removetree=false ;;
	-v)	set -x ;;
	*)	break ;;
	esac
	shift
done

# hope i got these right
case "$(uname -s)" in
NetBSD)	make=make ;;
*)	make=bmake ;;
esac

# check we can write in $cwd
mkdir -p packages || die "can't create files in $(pwd)"
echo "# pkgsrc.conf by ${USER} $(date)" > pkgsrc.conf
echo '.sinclude "/etc/mk.conf"' >> pkgsrc.conf
echo "USE_DESTDIR=yes" >> pkgsrc.conf

# get the mk tree
cvs -d ${repo} co -P pkgsrc/mk

# get package and dependency tree too
for d in $*; do
	getentry ${d}
done

# make it
made=""
notmade=""
makeconf="$(pwd)/pkgsrc.conf"
ex=0
for d in $*; do
	(cd pkgsrc/$d && env MAKECONF=${makeconf} ${make} install) || ex=1
done

# clean up
rm -f pkgsrc.conf
# if there is nothing to move, we already have the files
mv pkgsrc/packages/* packages/ 2>/dev/null
mv pkgsrc/distfiles/* distfiles/ 2>/dev/null
case $ex in
0)	if $removetree; then
		rm -rf pkgsrc
	fi ;;
esac

exit ${ex}
